@@ -17,7 +17,6 @@
 #include "soup-message.h"
 #include "soup-session.h"
 #include "soup-session-feature.h"
-#include "soup-socket.h"
 #include "soup-uri.h"
 
 /**
@@ -33,7 +32,7 @@
  * optionally configure it with soup_logger_set_request_filter(),
  * soup_logger_set_response_filter(), and soup_logger_set_printer(),
  * and then attach it to a session (or multiple sessions) with
- * soup_session_add_feature().
+ * soup_logger_attach().
  *
  * By default, the debugging output is sent to %stdout, and looks
  * something like:
@@ -67,18 +66,6 @@
  * across the lifetime of the #SoupLogger. In particular, this can be
  * used to identify when multiple messages are sent across the same
  * connection.
- *
- * Currently, the request half of the message is logged just before
- * the first byte of the request gets written to the network (from the
- * #SoupSession::request_started signal), which means that if you have
- * not made the complete request body available at that point, it will
- * not be logged. The response is logged just after the last byte of
- * the response body is read from the network (from the
- * #SoupMessage::got_body or #SoupMessage::got_informational signal),
- * which means that the #SoupMessage::got_headers signal, and anything
- * triggered off it (such as #SoupSession::authenticate) will be
- * emitted <emphasis>before</emphasis> the response headers are
- * actually logged.
  **/
 
 static void soup_logger_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
@@ -92,7 +79,7 @@ static void request_unqueued  (SoupSessionFeature *feature,
 
 G_DEFINE_TYPE_WITH_CODE (SoupLogger, soup_logger, G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (SOUP_TYPE_SESSION_FEATURE,
-						soup_logger_session_feature_init))
+						soup_logger_session_feature_init));
 
 typedef struct {
 	/* We use a mutex so that if requests are being run in
@@ -489,12 +476,8 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 	if (log_level == SOUP_LOGGER_LOG_MINIMAL)
 		return;
 
-	if (!soup_message_headers_get_one (msg->request_headers, "Host")) {
-		soup_logger_print (logger, SOUP_LOGGER_LOG_HEADERS, '>',
-				   "Host: %s%c%u", uri->host,
-				   soup_uri_uses_default_port (uri) ? '\0' : ':',
-				   uri->port);
-	}
+	soup_logger_print (logger, SOUP_LOGGER_LOG_HEADERS, '>',
+			   "Host: %s", uri->host);
 	soup_message_headers_iter_init (&iter, msg->request_headers);
 	while (soup_message_headers_iter_next (&iter, &name, &value)) {
 		if (!g_ascii_strcasecmp (name, "Authorization") &&
@@ -508,12 +491,10 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 	if (log_level == SOUP_LOGGER_LOG_HEADERS)
 		return;
 
-	if (msg->request_body->length &&
-	    soup_message_body_get_accumulate (msg->request_body)) {
+	if (msg->request_body->length) {
 		SoupBuffer *request;
 
 		request = soup_message_body_flatten (msg->request_body);
-		g_return_if_fail (request != NULL);
 		soup_buffer_free (request);
 
 		if (soup_message_headers_get_expectations (msg->request_headers) != SOUP_EXPECTATION_CONTINUE) {
@@ -564,7 +545,7 @@ print_response (SoupLogger *logger, SoupMessage *msg)
 	if (log_level == SOUP_LOGGER_LOG_HEADERS)
 		return;
 
-	if (msg->response_body->data) {
+	if (msg->response_body->length) {
 		soup_logger_print (logger, SOUP_LOGGER_LOG_BODY, '<',
 				   "\n%s", msg->response_body->data);
 	}
@@ -622,8 +603,6 @@ static void
 request_queued (SoupSessionFeature *logger, SoupSession *session,
 		SoupMessage *msg)
 {
-	g_return_if_fail (SOUP_IS_MESSAGE (msg));
-
 	g_signal_connect (msg, "got-informational",
 			  G_CALLBACK (got_informational),
 			  logger);
@@ -640,10 +619,6 @@ request_started (SoupSessionFeature *feature, SoupSession *session,
 	gboolean restarted;
 	guint msg_id;
 
-	g_return_if_fail (SOUP_IS_SESSION (session));
-	g_return_if_fail (SOUP_IS_MESSAGE (msg));
-	g_return_if_fail (SOUP_IS_SOCKET (socket));
-
 	msg_id = soup_logger_get_id (logger, msg);
 	if (msg_id)
 		restarted = TRUE;
@@ -666,8 +641,6 @@ static void
 request_unqueued (SoupSessionFeature *logger, SoupSession *session,
 		  SoupMessage *msg)
 {
-	g_return_if_fail (SOUP_IS_MESSAGE (msg));
-
 	g_signal_handlers_disconnect_by_func (msg, got_informational, logger);
 	g_signal_handlers_disconnect_by_func (msg, got_body, logger);
 }
