@@ -35,18 +35,7 @@
  * soup_socket_get_remote_address()) may be useful to applications.
  **/
 
-static void
-_soup_networking_init (void)
-{
-#ifdef G_OS_WIN32
-  WSADATA wsadata;
-  if (WSAStartup (MAKEWORD (2, 0), &wsadata) != 0)
-    g_error ("Windows Sockets could not be initialized");
-#endif
-}
-
-G_DEFINE_TYPE_WITH_CODE (SoupSocket, soup_socket, G_TYPE_OBJECT,
-			 _soup_networking_init ();)
+G_DEFINE_TYPE (SoupSocket, soup_socket, G_TYPE_OBJECT)
 
 enum {
 	READABLE,
@@ -79,14 +68,11 @@ typedef struct {
 
 	guint non_blocking:1;
 	guint is_server:1;
-	guint timed_out:1;
 	gpointer ssl_creds;
 
 	GMainContext   *async_context;
 	GSource        *watch_src;
 	GSource        *read_src, *write_src;
-	GSource        *read_timeout, *write_timeout;
-	GSource        *connect_timeout;
 	GByteArray     *read_buf;
 
 	GMutex *iolock, *addrlock;
@@ -94,6 +80,12 @@ typedef struct {
 } SoupSocketPrivate;
 #define SOUP_SOCKET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_SOCKET, SoupSocketPrivate))
 
+#ifdef HAVE_IPV6
+#define soup_sockaddr_max sockaddr_in6
+#else
+#define soup_sockaddr_max sockaddr_in
+#endif
+
 static void set_property (GObject *object, guint prop_id,
 			  const GValue *value, GParamSpec *pspec);
 static void get_property (GObject *object, guint prop_id,
@@ -137,14 +129,6 @@ disconnect_internal (SoupSocketPrivate *priv)
 		g_source_destroy (priv->write_src);
 		priv->write_src = NULL;
 	}
-	if (priv->read_timeout) {
-		g_source_destroy (priv->read_timeout);
-		priv->read_timeout = NULL;
-	}
-	if (priv->write_timeout) {
-		g_source_destroy (priv->write_timeout);
-		priv->write_timeout = NULL;
-	}
 }
 
 static void
@@ -162,8 +146,6 @@ finalize (GObject *object)
 
 	if (priv->watch_src)
 		g_source_destroy (priv->watch_src);
-	if (priv->connect_timeout)
-		g_source_destroy (priv->connect_timeout);
 	if (priv->async_context)
 		g_main_context_unref (priv->async_context);
 
@@ -199,8 +181,7 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 	 * @sock: the socket
 	 *
 	 * Emitted when an async socket is readable. See
-	 * soup_socket_read(), soup_socket_read_until() and
-	 * #SoupSocket:non-blocking.
+	 * soup_socket_read() and soup_socket_read_until().
 	 **/
 	signals[READABLE] =
 		g_signal_new ("readable",
@@ -216,7 +197,7 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 	 * @sock: the socket
 	 *
 	 * Emitted when an async socket is writable. See
-	 * soup_socket_write() and #SoupSocket:non-blocking.
+	 * soup_socket_write().
 	 **/
 	signals[WRITABLE] =
 		g_signal_new ("writable",
@@ -265,12 +246,6 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 			      SOUP_TYPE_SOCKET);
 
 	/* properties */
-	/**
-	 * SOUP_SOCKET_LOCAL_ADDRESS:
-	 *
-	 * Alias for the #SoupSocket:local-address property. (Address
-	 * of local end of socket.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_LOCAL_ADDRESS,
 		g_param_spec_object (SOUP_SOCKET_LOCAL_ADDRESS,
@@ -278,12 +253,6 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 				     "Address of local end of socket",
 				     SOUP_TYPE_ADDRESS,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	/**
-	 * SOUP_SOCKET_REMOTE_ADDRESS:
-	 *
-	 * Alias for the #SoupSocket:remote-address property. (Address
-	 * of remote end of socket.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_REMOTE_ADDRESS,
 		g_param_spec_object (SOUP_SOCKET_REMOTE_ADDRESS,
@@ -291,36 +260,6 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 				     "Address of remote end of socket",
 				     SOUP_TYPE_ADDRESS,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	/**
-	 * SoupSocket:non-blocking:
-	 *
-	 * Whether or not the socket uses non-blocking I/O.
-	 *
-	 * #SoupSocket's I/O methods are designed around the idea of
-	 * using a single codepath for both synchronous and
-	 * asynchronous I/O. If you want to read off a #SoupSocket,
-	 * the "correct" way to do it is to call soup_socket_read() or
-	 * soup_socket_read_until() repeatedly until you have read
-	 * everything you want. If it returns %SOUP_SOCKET_WOULD_BLOCK
-	 * at any point, stop reading and wait for it to emit the
-	 * #SoupSocket::readable signal. Then go back to the
-	 * reading-as-much-as-you-can loop. Likewise, for writing to a
-	 * #SoupSocket, you should call soup_socket_write() either
-	 * until you have written everything, or it returns
-	 * %SOUP_SOCKET_WOULD_BLOCK (in which case you wait for
-	 * #SoupSocket::writable and then go back into the loop).
-	 *
-	 * Code written this way will work correctly with both
-	 * blocking and non-blocking sockets; blocking sockets will
-	 * simply never return %SOUP_SOCKET_WOULD_BLOCK, and so the
-	 * code that handles that case just won't get used for them.
-	 **/
-	/**
-	 * SOUP_SOCKET_FLAG_NONBLOCKING:
-	 *
-	 * Alias for the #SoupSocket:non-blocking property. (Whether
-	 * or not the socket uses non-blocking I/O.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_NON_BLOCKING,
 		g_param_spec_boolean (SOUP_SOCKET_FLAG_NONBLOCKING,
@@ -328,12 +267,6 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 				      "Whether or not the socket uses non-blocking I/O",
 				      TRUE,
 				      G_PARAM_READWRITE));
-	/**
-	 * SOUP_SOCKET_IS_SERVER:
-	 *
-	 * Alias for the #SoupSocket:is-server property. (Whether or
-	 * not the socket is a server socket.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_IS_SERVER,
 		g_param_spec_boolean (SOUP_SOCKET_IS_SERVER,
@@ -341,24 +274,12 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 				      "Whether or not the socket is a server socket",
 				      FALSE,
 				      G_PARAM_READABLE));
-	/**
-	 * SOUP_SOCKET_SSL_CREDENTIALS:
-	 *
-	 * Alias for the #SoupSocket:ssl-credentials property.
-	 * (SSL credential information.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_SSL_CREDENTIALS,
 		g_param_spec_pointer (SOUP_SOCKET_SSL_CREDENTIALS,
 				      "SSL credentials",
 				      "SSL credential information, passed from the session to the SSL implementation",
 				      G_PARAM_READWRITE));
-	/**
-	 * SOUP_SOCKET_ASYNC_CONTEXT:
-	 *
-	 * Alias for the #SoupSocket:async-context property. (The
-	 * socket's #GMainContext.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_ASYNC_CONTEXT,
 		g_param_spec_pointer (SOUP_SOCKET_ASYNC_CONTEXT,
@@ -366,12 +287,6 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 				      "The GMainContext to dispatch this socket's async I/O in",
 				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-	/**
-	 * SOUP_SOCKET_TIMEOUT:
-	 *
-	 * Alias for the #SoupSocket:timeout property. (The timeout
-	 * in seconds for blocking socket I/O operations.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_TIMEOUT,
 		g_param_spec_uint (SOUP_SOCKET_TIMEOUT,
@@ -379,6 +294,11 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 				   "Value in seconds to timeout a blocking I/O",
 				   0, G_MAXUINT, 0,
 				   G_PARAM_READWRITE));
+
+#ifdef G_OS_WIN32
+	/* Make sure WSAStartup() gets called. */
+	soup_address_get_type ();
+#endif
 }
 
 
@@ -413,8 +333,8 @@ static void
 set_fdflags (SoupSocketPrivate *priv)
 {
 	int opt;
-#ifndef G_OS_WIN32
 	struct timeval timeout;
+#ifndef G_OS_WIN32
 	int flags;
 #endif
 
@@ -437,7 +357,6 @@ set_fdflags (SoupSocketPrivate *priv)
 	setsockopt (priv->sockfd, SOL_SOCKET,
 		    SO_REUSEADDR, (void *) &opt, sizeof (opt));
 
-#ifndef G_OS_WIN32
 	timeout.tv_sec = priv->timeout;
 	timeout.tv_usec = 0;
 	setsockopt (priv->sockfd, SOL_SOCKET,
@@ -447,18 +366,6 @@ set_fdflags (SoupSocketPrivate *priv)
 	timeout.tv_usec = 0;
 	setsockopt (priv->sockfd, SOL_SOCKET,
 		    SO_SNDTIMEO, (void *) &timeout, sizeof (timeout));
-#else
-	if (priv->timeout < G_MAXINT / 1000)
-		opt = priv->timeout * 1000;
-	else
-		opt = 0;
-
-	setsockopt (priv->sockfd, SOL_SOCKET,
-		    SO_RCVTIMEO, (void *) &opt, sizeof (opt));
-	
-	setsockopt (priv->sockfd, SOL_SOCKET,
-		    SO_SNDTIMEO, (void *) &opt, sizeof (opt));
-#endif
 
 #ifndef G_OS_WIN32
 	priv->iochannel =
@@ -607,10 +514,6 @@ connect_watch (GIOChannel* iochannel, GIOCondition condition, gpointer data)
 	/* Remove the watch now in case we don't return immediately */
 	g_source_destroy (priv->watch_src);
 	priv->watch_src = NULL;
-	if (priv->connect_timeout) {
-		g_source_destroy (priv->connect_timeout);
-		priv->connect_timeout = NULL;
-	}
 
 	if ((condition & ~(G_IO_IN | G_IO_OUT)) ||
 	    (getsockopt (priv->sockfd, SOL_SOCKET, SO_ERROR,
@@ -621,22 +524,6 @@ connect_watch (GIOChannel* iochannel, GIOCondition condition, gpointer data)
 	return idle_connect_result (sacd);
 }
 
-static gboolean
-connect_timeout (gpointer data)
-{
-	SoupSocketAsyncConnectData *sacd = data;
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sacd->sock);
-
-	/* Remove the watch now in case we don't return immediately */
-	g_source_destroy (priv->watch_src);
-	priv->watch_src = NULL;
-	g_source_destroy (priv->connect_timeout);
-	priv->connect_timeout = NULL;
-
-	disconnect_internal (priv);
-	return idle_connect_result (sacd);
-}
-
 static void
 got_address (SoupAddress *addr, guint status, gpointer user_data)
 {
@@ -662,8 +549,8 @@ async_cancel (GCancellable *cancellable, gpointer user_data)
 	if (priv->watch_src)
 		g_source_destroy (priv->watch_src);
 	disconnect_internal (priv);
-	priv->watch_src = soup_add_completion (priv->async_context,
-					       idle_connect_result, sacd);
+	priv->watch_src = soup_add_idle (priv->async_context,
+					 idle_connect_result, sacd);
 }
 
 static guint
@@ -754,12 +641,6 @@ soup_socket_connect_async (SoupSocket *sock, GCancellable *cancellable,
 					   G_IO_PRI | G_IO_ERR |
 					   G_IO_HUP | G_IO_NVAL,
 					   connect_watch, sacd);
-		if (priv->timeout) {
-			priv->connect_timeout =
-				soup_add_timeout (priv->async_context,
-						  priv->timeout * 1000,
-						  connect_timeout, sacd);
-		}
 		if (cancellable) {
 			sacd->cancel_id =
 				g_signal_connect (cancellable, "cancelled",
@@ -767,8 +648,8 @@ soup_socket_connect_async (SoupSocket *sock, GCancellable *cancellable,
 						  sacd);
 		}
 	} else {
-		priv->watch_src = soup_add_completion (priv->async_context,
-						       idle_connect_result, sacd);
+		priv->watch_src = soup_add_idle (priv->async_context,
+						 idle_connect_result, sacd);
 	}
 }
 
@@ -831,20 +712,12 @@ soup_socket_connect_sync (SoupSocket *sock, GCancellable *cancellable)
 	return status;
 }
 
-int
-soup_socket_get_fd (SoupSocket *sock)
-{
-	g_return_val_if_fail (SOUP_IS_SOCKET (sock), -1);
-
-	return SOUP_SOCKET_GET_PRIVATE (sock)->sockfd;
-}
-
 static gboolean
 listen_watch (GIOChannel* iochannel, GIOCondition condition, gpointer data)
 {
 	SoupSocket *sock = data, *new;
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock), *new_priv;
-	struct sockaddr_storage sa;
+	struct soup_sockaddr_max sa;
 	int sa_len, sockfd;
 
 	if (condition & (G_IO_HUP | G_IO_ERR)) {
@@ -1058,11 +931,6 @@ soup_socket_disconnect (SoupSocket *sock)
 	if (already_disconnected)
 		return;
 
-	/* Keep ref around signals in case the object is unreferenced
-	 * in a handler
-	 */
-	g_object_ref (sock);
-
 	/* Give all readers a chance to notice the connection close */
 	g_signal_emit (sock, signals[READABLE], 0);
 
@@ -1070,8 +938,6 @@ soup_socket_disconnect (SoupSocket *sock)
 
 	/* Then let everyone know we're disconnected */
 	g_signal_emit (sock, signals[DISCONNECTED], 0);
-
-	g_object_unref (sock);
 }
 
 /**
@@ -1111,7 +977,7 @@ soup_socket_get_local_address (SoupSocket *sock)
 
 	g_mutex_lock (priv->addrlock);
 	if (!priv->local_addr) {
-		struct sockaddr_storage bound_sa;
+		struct soup_sockaddr_max bound_sa;
 		int sa_len;
 
 		sa_len = sizeof (bound_sa);
@@ -1141,7 +1007,7 @@ soup_socket_get_remote_address (SoupSocket *sock)
 
 	g_mutex_lock (priv->addrlock);
 	if (!priv->remote_addr) {
-		struct sockaddr_storage bound_sa;
+		struct soup_sockaddr_max bound_sa;
 		int sa_len;
 
 		sa_len = sizeof (bound_sa);
@@ -1154,29 +1020,7 @@ soup_socket_get_remote_address (SoupSocket *sock)
 }
 
 
-static gboolean
-socket_timeout (gpointer sock)
-{
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
-	gboolean readable = FALSE, writable = FALSE;
 
-	priv->timed_out = TRUE;
-	if (priv->read_timeout) {
-		priv->read_timeout = NULL;
-		readable = TRUE;
-	}
-	if (priv->write_timeout) {
-		priv->write_timeout = NULL;
-		writable = TRUE;
-	}
-
-	if (readable)
-		g_signal_emit (sock, signals[READABLE], 0);
-	if (writable)
-		g_signal_emit (sock, signals[WRITABLE], 0);
-
-	return FALSE;
-}
 
 static gboolean
 socket_read_watch (GIOChannel *chan, GIOCondition cond, gpointer user_data)
@@ -1185,10 +1029,6 @@ socket_read_watch (GIOChannel *chan, GIOCondition cond, gpointer user_data)
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
 	priv->read_src = NULL;
-	if (priv->read_timeout) {
-		g_source_destroy (priv->read_timeout);
-		priv->read_timeout = NULL;
-	}
 
 	if (cond & (G_IO_ERR | G_IO_HUP))
 		soup_socket_disconnect (sock);
@@ -1212,12 +1052,6 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
 	if (!priv->iochannel)
 		return SOUP_SOCKET_EOF;
 
-	if (priv->timed_out) {
-		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
-				     "Timed out");
-		return SOUP_SOCKET_ERROR;
-	}
-
 	status = g_io_channel_read_chars (priv->iochannel,
 					  buffer, len, nread, &my_err);
 	if (my_err) {
@@ -1239,11 +1073,8 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
 		 * a socket timeout and should be treated as an error
 		 * condition.
 		 */
-		if (!priv->non_blocking) {
-			g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
-					     "Timed out");
+		if (!priv->non_blocking)
 			return SOUP_SOCKET_ERROR;
-		}
 
 		if (!priv->read_src) {
 			priv->read_src =
@@ -1251,12 +1082,6 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
 						   priv->iochannel,
 						   cond | G_IO_HUP | G_IO_ERR,
 						   socket_read_watch, sock);
-			if (priv->timeout) {
-				priv->read_timeout =
-					soup_add_timeout (priv->async_context,
-							  priv->timeout * 1000,
-							  socket_timeout, sock);
-			}
 		}
 		g_clear_error (error);
 		return SOUP_SOCKET_WOULD_BLOCK;
@@ -1313,15 +1138,14 @@ read_from_buf (SoupSocket *sock, gpointer buffer, gsize len, gsize *nread)
  * Attempts to read up to @len bytes from @sock into @buffer. If some
  * data is successfully read, soup_socket_read() will return
  * %SOUP_SOCKET_OK, and *@nread will contain the number of bytes
- * actually read (which may be less than @len).
+ * actually read.
  *
  * If @sock is non-blocking, and no data is available, the return
  * value will be %SOUP_SOCKET_WOULD_BLOCK. In this case, the caller
- * can connect to the #SoupSocket::readable signal to know when there
- * is more data to read. (NB: You MUST read all available data off the
- * socket first. #SoupSocket::readable is only emitted after
- * soup_socket_read() returns %SOUP_SOCKET_WOULD_BLOCK, and it is only
- * emitted once. See the documentation for #SoupSocket:non-blocking.)
+ * can connect to the %readable signal to know when there is more data
+ * to read. (NB: You MUST read all available data off the socket
+ * first. The %readable signal will only be emitted after
+ * soup_socket_read() has returned %SOUP_SOCKET_WOULD_BLOCK.)
  *
  * Return value: a #SoupSocketIOStatus, as described above (or
  * %SOUP_SOCKET_EOF if the socket is no longer connected, or
@@ -1368,14 +1192,6 @@ soup_socket_read (SoupSocket *sock, gpointer buffer, gsize len,
  * included in the returned data, and *@got_boundary will be set to
  * %TRUE.) Any data after the boundary will returned in future reads.
  *
- * soup_socket_read_until() will almost always return fewer than @len
- * bytes: if the boundary is found, then it will only return the bytes
- * up until the end of the boundary, and if the boundary is not found,
- * then it will leave the last <literal>(boundary_len - 1)</literal>
- * bytes in its internal buffer, in case they form the start of the
- * boundary string. Thus, @len normally needs to be at least 1 byte
- * longer than @boundary_len if you want to make any progress at all.
- *
  * Return value: as for soup_socket_read()
  **/
 SoupSocketIOStatus
@@ -1446,10 +1262,6 @@ socket_write_watch (GIOChannel *chan, GIOCondition cond, gpointer user_data)
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
 	priv->write_src = NULL;
-	if (priv->write_timeout) {
-		g_source_destroy (priv->write_timeout);
-		priv->write_timeout = NULL;
-	}
 
 	if (cond & (G_IO_ERR | G_IO_HUP))
 		soup_socket_disconnect (sock);
@@ -1469,17 +1281,15 @@ socket_write_watch (GIOChannel *chan, GIOCondition cond, gpointer user_data)
  * @error: error pointer
  *
  * Attempts to write @len bytes from @buffer to @sock. If some data is
- * successfully written, the return status will be %SOUP_SOCKET_OK,
- * and *@nwrote will contain the number of bytes actually written
- * (which may be less than @len).
+ * successfully written, the resturn status will be
+ * %SOUP_SOCKET_OK, and *@nwrote will contain the number of bytes
+ * actually written.
  *
  * If @sock is non-blocking, and no data could be written right away,
  * the return value will be %SOUP_SOCKET_WOULD_BLOCK. In this case,
- * the caller can connect to the #SoupSocket::writable signal to know
- * when more data can be written. (NB: #SoupSocket::writable is only
- * emitted after soup_socket_write() returns %SOUP_SOCKET_WOULD_BLOCK,
- * and it is only emitted once. See the documentation for
- * #SoupSocket:non-blocking.)
+ * the caller can connect to the %writable signal to know when more
+ * data can be written. (NB: %writable is only emitted after a
+ * %SOUP_SOCKET_WOULD_BLOCK.)
  *
  * Return value: a #SoupSocketIOStatus, as described above (or
  * %SOUP_SOCKET_EOF or %SOUP_SOCKET_ERROR. @error will be set if the
@@ -1506,12 +1316,6 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 		g_mutex_unlock (priv->iolock);
 		return SOUP_SOCKET_EOF;
 	}
-	if (priv->timed_out) {
-		g_mutex_unlock (priv->iolock);
-		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
-				     "Timed out");
-		return SOUP_SOCKET_ERROR;
-	}
 	if (priv->write_src) {
 		g_mutex_unlock (priv->iolock);
 		return SOUP_SOCKET_WOULD_BLOCK;
@@ -1531,8 +1335,6 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 	 */
 	if (!priv->non_blocking && status == G_IO_STATUS_AGAIN) {
 		g_mutex_unlock (priv->iolock);
-		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
-				     "Timed out");
 		return SOUP_SOCKET_ERROR;
 	}
 
@@ -1553,12 +1355,6 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 				   priv->iochannel,
 				   cond | G_IO_HUP | G_IO_ERR, 
 				   socket_write_watch, sock);
-	if (priv->timeout) {
-		priv->write_timeout = soup_add_timeout (priv->async_context,
-							priv->timeout * 1000,
-							socket_timeout, sock);
-	}
-
 	g_mutex_unlock (priv->iolock);
 	return SOUP_SOCKET_WOULD_BLOCK;
 }
