@@ -33,7 +33,7 @@ typedef enum {
 	NTLM_RECEIVED_REQUEST,
 	NTLM_SENT_CHALLENGE,
 	NTLM_AUTHENTICATED_ALICE,
-	NTLM_AUTHENTICATED_BOB
+	NTLM_AUTHENTICATED_BOB,
 } NTLMServerState;
 
 #define NTLM_REQUEST_START "TlRMTVNTUAABAAAA"
@@ -87,8 +87,7 @@ server_callback (SoupServer *server, SoupMessage *msg,
 
 	socket = soup_client_context_get_socket (client);
 	state = GPOINTER_TO_INT (g_hash_table_lookup (connections, socket));
-	auth = soup_message_headers_get_one (msg->request_headers,
-					     "Authorization");
+	auth = soup_message_headers_get (msg->request_headers, "Authorization");
 
 	if (auth) {
 		if (!strncmp (auth, "NTLM ", 5)) {
@@ -111,7 +110,6 @@ server_callback (SoupServer *server, SoupMessage *msg,
 			if (!strncmp (decoded, "alice:password", len) ||
 			    !strncmp (decoded, "bob:password", len))
 				auth_required = FALSE;
-			g_free (decoded);
 		}
 	}
 
@@ -176,8 +174,8 @@ prompt_check (SoupMessage *msg, gpointer user_data)
 	NTLMState *state = user_data;
 	const char *header;
 
-	header = soup_message_headers_get_list (msg->response_headers,
-						"WWW-Authenticate");
+	header = soup_message_headers_get (msg->response_headers,
+					   "WWW-Authenticate");
 	if (header && strstr (header, "Basic "))
 		state->got_basic_prompt = TRUE;
 	if (!state->sent_ntlm_request) {
@@ -193,8 +191,8 @@ challenge_check (SoupMessage *msg, gpointer user_data)
 	NTLMState *state = user_data;
 	const char *header;
 
-	header = soup_message_headers_get_list (msg->response_headers,
-						"WWW-Authenticate");
+	header = soup_message_headers_get (msg->response_headers,
+					    "WWW-Authenticate");
 	if (header && !strncmp (header, "NTLM ", 5))
 		state->got_ntlm_challenge = TRUE;
 }
@@ -205,8 +203,8 @@ request_check (SoupMessage *msg, gpointer user_data)
 	NTLMState *state = user_data;
 	const char *header;
 
-	header = soup_message_headers_get_one (msg->request_headers,
-					       "Authorization");
+	header = soup_message_headers_get (msg->request_headers,
+					   "Authorization");
 	if (header && !strncmp (header, "NTLM " NTLM_REQUEST_START,
 				strlen ("NTLM " NTLM_REQUEST_START)))
 		state->sent_ntlm_request = TRUE;
@@ -218,8 +216,8 @@ response_check (SoupMessage *msg, gpointer user_data)
 	NTLMState *state = user_data;
 	const char *header;
 
-	header = soup_message_headers_get_one (msg->request_headers,
-					       "Authorization");
+	header = soup_message_headers_get (msg->request_headers,
+					   "Authorization");
 	if (header && !strncmp (header, "NTLM " NTLM_RESPONSE_START,
 				strlen ("NTLM " NTLM_RESPONSE_START)))
 		state->sent_ntlm_response = TRUE;
@@ -387,7 +385,8 @@ do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm, const char *user)
 		    user != NULL ? SOUP_STATUS_OK :
 		    SOUP_STATUS_UNAUTHORIZED);
 
-	soup_test_session_abort_unref (session);
+	soup_session_abort (session);
+	g_object_unref (session);
 }
 
 static void
@@ -420,7 +419,7 @@ main (int argc, char **argv)
 
 	loop = g_main_loop_new (NULL, TRUE);
 
-	uri = soup_uri_new ("http://127.0.0.1/");
+	uri = soup_uri_new ("http://localhost/");
 	soup_uri_set_port (uri, soup_server_get_port (server));
 	do_ntlm_tests (uri);
 	soup_uri_free (uri);
